@@ -21,7 +21,7 @@ MA 02110-1301 USA
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.39 2006/03/31 19:14:34 evertonm Exp $
+  $Id: main.c,v 1.40 2006/04/07 21:46:21 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -98,6 +98,7 @@ static void usage(FILE *out, const char *prog_name)
           "  -4             disable IPv4\n"
 	  "  -q             quietly hide soft read/write errors\n"
 	  "  -y tos         ip tos (defaults to %d, tos=-1 means don't change)\n"
+	  "  -K udp-ka-time UDP keepalive timeout in ms (defaults to %ld ms)\n"
           "  -x [win]       experimental UDP reception statistics (win=%d)\n"
           "  -L loss        experimental UDP sending loss\n"
           "  -D dup         experimental UDP sending duplication\n"
@@ -141,6 +142,7 @@ static void usage(FILE *out, const char *prog_name)
           nepim_global.udp_write_size,
           nepim_global.udp_read_size,
 	  nepim_global.tos,
+	  (long) nepim_global.udp_keepalive_timer,
           nepim_global.udp_win_max,
           nepim_global.pipes,
           bit_rate,
@@ -212,6 +214,17 @@ int main(int argc, const char *argv[])
       continue;
     }
 
+    if (!strcmp(arg, "-K")) {
+      ++i;
+      if (i >= argc) {
+        fprintf(stderr, "%s: missing keepalive timeout\n",
+                nepim_global.prog_name);
+        exit(1);
+      }
+      nepim_global.udp_keepalive_timer = atoi(argv[i]);
+      continue;
+    }
+
     if (!strcmp(arg, "-q")) {
       nepim_global.soft_error_verbose = 0;
       continue;
