@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: server.c,v 1.46 2005/12/20 09:36:09 evertonm Exp $ */
+/* $Id: server.c,v 1.50 2006/03/31 18:52:52 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,6 +30,9 @@ MA 02110-1301 USA
 #include <errno.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <sys/un.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #include "conf.h"
 #include "sock.h"
@@ -611,10 +614,12 @@ static void *on_tcp_connect(oop_source *src, int sd,
   union {
     struct sockaddr_in inet;
     struct sockaddr_in6 inet6;
+    struct sockaddr_un un;
   } sa;
   int len = sizeof(sa);
   int result;
-  char addr_buf[100];
+  int family;
+  char addr_buf[500];
 
   conn_sd = accept(sd, (struct sockaddr *) &sa, &len);
   if (conn_sd < 0) {
@@ -625,12 +630,20 @@ static void *on_tcp_connect(oop_source *src, int sd,
   }
 
   nepim_sock_dump_addr(addr_buf, sizeof(addr_buf),
-                       (const struct sockaddr *) &sa);
+                       (const struct sockaddr *) &sa,
+		       len);
 
-  fprintf(stderr,
-          "%d: TCP incoming: %s,%d\n",
-          conn_sd, addr_buf, 
-          nepim_sock_get_port((const struct sockaddr *) &sa));
+  family = nepim_sock_family((const struct sockaddr *) &sa);
+
+  if (family == AF_UNIX)
+    fprintf(stderr,
+	    "%d: UNIX incoming: path=%s (sockaddr_len=%d path_len=%d)\n",
+	    conn_sd, addr_buf, len, strlen(addr_buf));
+    else
+      fprintf(stderr,
+	      "%d: TCP incoming: %s,%d\n",
+	      conn_sd, addr_buf, 
+	      nepim_sock_get_port((const struct sockaddr *) &sa));
 
   if (nepim_global.tcpwrap) {
     if (!nepim_hosts_ctl(nepim_global.tcpwrap,
@@ -646,7 +659,7 @@ static void *on_tcp_connect(oop_source *src, int sd,
   }
 
   result = nepim_socket_opt(conn_sd, nepim_global.pmtu_mode, 
-                            nepim_global.ttl);
+                            nepim_global.ttl, nepim_global.tos);
   if (result) {
     fprintf(stderr, 
             "%d: %s: %s: could not set socket options: %d\n",
@@ -655,7 +668,7 @@ static void *on_tcp_connect(oop_source *src, int sd,
     return OOP_CONTINUE;
   }
 
-  result = nepim_socket_tcp_opt(conn_sd);
+  result = nepim_socket_tcp_opt(conn_sd, family != PF_UNIX /* nodelay */);
   if (result) {
     fprintf(stderr, 
             "%d: %s: %s: could not set tcp options: %d\n",
@@ -710,12 +723,12 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
   hints.ai_addrlen = 0;
   hints.ai_addr = 0;
   hints.ai_canonname = 0;
-
+  
   result = getaddrinfo(hostname, portname, &hints, &ai_res);
   if (result) {
     fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
-            __PRETTY_FUNCTION__,
-            hostname, portname, gai_strerror(result));
+	    __PRETTY_FUNCTION__,
+	    hostname, portname, gai_strerror(result));
     return 0;
   }
 
@@ -743,7 +756,9 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
                                       nepim_global.pmtu_mode,
                                       nepim_global.ttl,
                                       nepim_global.win_recv,
-                                      nepim_global.win_send);
+                                      nepim_global.win_send,
+				      -1 /* tcp nodelay */,
+				      nepim_global.tos);
     if (sd < 0) {
       fprintf(stderr, 
               "%s: TCP listener socket failed for %s,%s(%d): %d\n",
@@ -768,6 +783,56 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
   return tcp_listeners;
 }
 
+static int spawn_unix_listener(const char *path)
+{
+  struct sockaddr_un un_sock_addr;
+  int len;
+  int sd;
+
+  len = strlen(path);
+  if (len >= (int) sizeof(un_sock_addr.sun_path)) {
+    fprintf(stderr, "%s %s: unix socket path overly long: len=%d > max=%d\n",
+	    __FILE__, __PRETTY_FUNCTION__,
+	    len, sizeof(un_sock_addr.sun_path) - 1);
+    return 0;
+  }
+  
+  memset(&un_sock_addr, 0, sizeof(un_sock_addr));
+  
+  un_sock_addr.sun_family = AF_UNIX;
+  strcpy(un_sock_addr.sun_path, path);
+  
+  sd = nepim_create_listener_socket((struct sockaddr *) &un_sock_addr, 
+				    sizeof(un_sock_addr),
+				    AF_UNIX,
+				    SOCK_STREAM, 
+				    0,
+				    nepim_global.listen_backlog,
+				    nepim_global.pmtu_mode,
+				    nepim_global.ttl,
+				    nepim_global.win_recv,
+				    nepim_global.win_send,
+				    0 /* tcp nodelay */,
+				    nepim_global.tos);
+  if (sd < 0) {
+    fprintf(stderr, 
+	    "%s %s: UNIX stream listener socket failed for path=%s\n",
+	    __FILE__, __PRETTY_FUNCTION__, path); 
+    return 0;
+  }
+
+  nepim_global.oop_src->on_fd(nepim_global.oop_src,
+			      sd, OOP_READ,
+			      on_tcp_connect, 0);
+
+  fprintf(stderr, 
+	  "%d: UNIX stream socket listening on path=%s\n",
+	  sd, path);
+
+  return 1;
+}
+
+
 extern int nepim_udp_listener(const char *hostname,
                               const char *portname, int join);
 
@@ -796,11 +861,61 @@ static void parse_listeners(const char *list, int *udp, int *tcp, int join)
   }
 }
 
+static int parse_unix_listeners(const char *list)
+{
+  int listeners = 0;
+  int i;
+
+  int size = addr_list_size(list);
+
+  for (i = 0; i < size; ++i) {
+    char buf[500];
+
+    if (addr_list_get(list, i, buf, sizeof(buf))) {
+      fprintf(stderr, 
+              "%s: failure parsing address %d/%d from list: %s\n",
+              nepim_global.prog_name, i, size, list);
+      continue;
+    }
+
+    listeners += spawn_unix_listener(buf);
+  }
+
+  return listeners;
+}
+
+extern int nepim_unix_dgram_listener(const char *path);
+
+static int parse_unix_dgram_listeners(const char *list)
+{
+  int listeners = 0;
+  int i;
+
+  int size = addr_list_size(list);
+
+  for (i = 0; i < size; ++i) {
+    char buf[500];
+
+    if (addr_list_get(list, i, buf, sizeof(buf))) {
+      fprintf(stderr, 
+              "%s: failure parsing address %d/%d from list: %s\n",
+              nepim_global.prog_name, i, size, list);
+      continue;
+    }
+
+    listeners += nepim_unix_dgram_listener(buf);
+  }
+
+  return listeners;
+}
+
 void nepim_server_run()
 {
   void *result;
   int tcp_listeners = 0;
   int udp_listeners = 0;
+  int unix_listeners;
+  int unix_dgram_listeners;
 
   /* these must be initialized before spawning listeners */
   nepim_pipe_set_init(&pipes);
@@ -815,7 +930,19 @@ void nepim_server_run()
     parse_listeners(nepim_global.join_list, &udp_listeners, &tcp_listeners,
                     1 /* do join */);
 
-  if (!nepim_global.bind_list && !nepim_global.join_list) {
+  if (nepim_global.unix_list)
+    unix_listeners = parse_unix_listeners(nepim_global.unix_list);
+
+  if (nepim_global.unix_dgram_list)
+    unix_dgram_listeners = parse_unix_dgram_listeners(nepim_global.unix_dgram_list);
+
+  /*
+    Listen on default wildcard addresses?
+  */
+  if (!nepim_global.bind_list &&
+      !nepim_global.join_list &&
+      !nepim_global.unix_list &&
+      !nepim_global.unix_dgram_list) {
     if (!nepim_global.no_inet6)
       tcp_listeners += spawn_tcp_listener(INET6_ANY, nepim_global.portname);
     tcp_listeners += spawn_tcp_listener(INET_ANY, nepim_global.portname);
@@ -826,7 +953,10 @@ void nepim_server_run()
                                         0 /* no join */);
   }
   
-  if (tcp_listeners || udp_listeners)
+  if (tcp_listeners ||
+      udp_listeners ||
+      unix_listeners ||
+      unix_dgram_listeners)
     fprintf(stderr, "%s: server ready\n", nepim_global.prog_name);
 
   result = oop_sys_run(nepim_global.oop_sys);
